// NpMsgrMain.h: interface for the CNpMsgrMain class.
//
//////////////////////////////////////////////////////////////////////
#pragma once

#include "INpMsgr.h"
#include "INpSession.h"


#include <string>
#include <vector>
#include <map>
#include <list>
using namespace std;


#include "GBStream.h"
#define __USE_CPP_STL
#include "ConvertUTF.h"

#include "NpMsgrConfig.h"
#include "NpMsgrUser.h"
#include "NpMsgrBuddy.h"
#include "NpMsgrIPT.h"

#if defined(WIN32)
#include "IkWnd.h"
#else
#include "IGTEventDelegate.h"
#endif
#include "Mutex.h"

#define UM_LOGIN_AUTH_RESULT		(WM_USER+1)
#define UM_LOGIN_GETDATA_RESULT		(WM_USER+2)


/**
	@ingroup	NpMsgr
	@brief		޽ ÷ Ŷ INpSessionDispatcher  Ŭ. Ŭ̾Ʈ ޽Ʈ Ŷ  Ѵ.
	@author		翵
*/
class CNpSessionDispatcher : public INpSessionDispatcher
{
public:
	CNpSessionDispatcher(INpMsgrUser * pNpMsgrUser) : m_pNpMsgrUser( pNpMsgrUser )
	{
		m_nTrId_Serial = 1;
		m_pMainServerSession = NULL;
		m_bClosedOnlineMode = false;
	}

	~CNpSessionDispatcher()
	{
		Clear_ClosedOnlineSendBuffer();
	}

	void CALLBACK OnReceivePacket(const INpPacket * pPacket);

	void StartMainServerSession( INpMsgrUser * pNpMsgrUser, INpSession * pMainServerSession, int nTrId )
	{
		m_bClosedOnlineMode = false;
		m_pNpMsgrUser = pNpMsgrUser;
		m_pMainServerSession = pMainServerSession;
		m_nTrId_Serial = nTrId;
	}

	void SetClosedOnlineMode()
	{ 
		m_bClosedOnlineMode = true;
		m_pMainServerSession = NULL;
	}

	bool GetClosedOnlineMode()
	{
		return m_bClosedOnlineMode;
	}

	void RestartMainServerSession( INpSession * pMainServerSession, int nTrId )
	{
		m_bClosedOnlineMode = false;
		m_pMainServerSession = pMainServerSession;
		m_nTrId_Serial = nTrId;

		Send_ClosedOnlineSendBuffer();
	}

	void EndMainServerSession()
	{
		m_bClosedOnlineMode = false;
		m_pNpMsgrUser = NULL;
		m_pMainServerSession = NULL;

		m_mapResTRIDHandler.clear();

		Clear_ClosedOnlineSendBuffer();
	}

	/////////////////////////////////////////////////////////////////////////////////
	// class INpSessionDispatcher
	virtual void Register_INCOMING_REQUEST_PACKET_Handler(USHORT usCMMD, _INpSessionDispatcher_PacketHandler * pHandler);
	virtual void Deregister_INCOMING_REQUEST_PACKET_Handler(USHORT usCMMD);
	
	virtual int Send_REQUEST_PACKET(USHORT usCMMD, USHORT usBodyLength, LPCSTR szBodyData, _INpSessionDispatcher_PacketHandler * pRESPONSE_PACKET_Handler = NULL);	// return trid
	virtual bool Deregister_RESPONSE_PACKET_Handler(int nTrId);
	
	virtual void Send_RESPONSE_PACKET(USHORT usCMMD, int nTrId, USHORT usBodyLength, LPCSTR szBodyData);	// return trid

	virtual void Register_CToC_PACKET_Handler(USHORT usCToC_SubCMMD, _INpSessionDispatcher_PacketHandler * pHandler);
	virtual void Deregister_CToC_PACKET_Handler(USHORT usCToC_SubCMMD, _INpSessionDispatcher_PacketHandler* pHandler);
	
	//virtual int  Send_CToC_PACKET(LPCWSTR wszToId, USHORT usBodyLength, LPCSTR szBodyData, _INpSessionDispatcher_PacketHandler * pRESPONSE_PACKET_Handler);	// return trid

	virtual void Send_PING(void);

	virtual bool PING(void);

protected:
	bool m_bClosedOnlineMode;
	INpMsgrUser * m_pNpMsgrUser;
	int m_nTrId_Serial;
	INpSession * m_pMainServerSession;

	map<int, _INpSessionDispatcher_PacketHandler*> m_mapResTRIDHandler;
	map<USHORT, _INpSessionDispatcher_PacketHandler*> m_mapReqCMMDHandler;
	map<USHORT, list<_INpSessionDispatcher_PacketHandler*> > m_mapCToC_SubCMMDHandler;

	class tCLOSED_ONLINE_SEND_BUFFER
	{
	public:
		int m_nTrId;
		char * m_pData;
		int m_nDataLen;
		USHORT m_usCMMD;
		WSTRING m_wstrToId;

		tCLOSED_ONLINE_SEND_BUFFER(USHORT usCMMD, int nTrId, const char * pData, int nDataLen)
		{
			////////////////////////////////////////////////
			// KEH_API
			// 10/06/24 
			//    . 
			// , nTrId, usCMMD    ߾. 
			// .. ̷ װ ֳ.. ̷ JS  HACK2 αװ ׿ ..
			m_nTrId = nTrId;
			m_usCMMD = usCMMD;
			////////////////////////////////////////////////
			m_pData = (char*)malloc( nDataLen + 1);
			if (NULL != m_pData)
			{
				m_nDataLen = nDataLen;

				memcpy( m_pData, pData, m_nDataLen );
				m_pData[m_nDataLen] = 0;	// αб ,, 
			}
			else
			{
				m_nDataLen = 0;
			}
		}

		~tCLOSED_ONLINE_SEND_BUFFER()
		{
			if (NULL != m_pData) free( m_pData );
		}
	};

	list<tCLOSED_ONLINE_SEND_BUFFER*> m_listClosedOnlineSendBuffer;

	void Clear_ClosedOnlineSendBuffer();
	void Send_ClosedOnlineSendBuffer();
};




/**
	@ingroup	NpMsgr
	@brief		Ŭ̾Ʈ ޽Ʈ  ̽ Ǵ INpMsgr   Ŭ. ޽ ý   پ ̽ Ŭ .
	@author		翵
*/
class CNpMsgrMain : public INpMsgr,
	public _CIkWndHandler,
	public _INpSessionEvents,
	public _INpSessionDispatcher_PacketHandler
{
public:
	CNpMsgrMain(_INpMsgrEvents * pEvents);
	virtual ~CNpMsgrMain();

	void Send_ADDNOTIFYBUDDY();	

	CNpMsgrUser * m_pNpMsgrUser;
	CNpMsgrConfig * m_pNpMsgrConfig;
	CNpMsgrBuddy * m_pNpMsgrBuddy;
	CNpMsgrIPT * m_pNpMsgrIPT;

	CNpSessionDispatcher * m_pNpSD_MainServer;
#if defined(WIN32)
	CIkWnd m_ikWnd;
	CIkWnd & WINDOW() { return m_ikWnd; }
#else
	IGTEventDelegate *m_pEventDelegate;
#endif
	string m_strLastErrorMsg;
	CNpMsgrUser * USER() { return m_pNpMsgrUser; }
	CNpMsgrConfig * CONFIG() { return m_pNpMsgrConfig; }
	CNpMsgrBuddy * BUDDY() { return m_pNpMsgrBuddy; }
	CNpMsgrIPT * IPT() { return m_pNpMsgrIPT; }
	
	CNpSessionDispatcher * MAINSERVER_SD() { return m_pNpSD_MainServer; }

	virtual UINT GetLRSCookie(){ return m_nLRSCookie; }

	////////////////////////////////////////////////////////////////////////////
	//
	_INpMsgrEvents * m_pEvents;

	/////////////////////////////////////////////////////////////////////////////////
	// INpMsgr
	virtual INpMsgrUser * GetNpMsgrUser() { return m_pNpMsgrUser; }
	virtual INpMsgrConfig * GetNpMsgrConfig() { return m_pNpMsgrConfig; }	
	virtual INpMsgrBuddy * GetNpMsgrBuddy() { return m_pNpMsgrBuddy; }
	virtual INpMsgrIPT * GetNpMsgrIPT() { return m_pNpMsgrIPT; }

	virtual INpSessionDispatcher * GetMainServer_SessionDispatcher() { return m_pNpSD_MainServer; }

	virtual void SetMobileInfo(short nOverwriteDataType, unsigned short nCountryCode, const char * strMobileNumber, const char * strMobileToken);

	virtual bool Login(LPCWSTR wszUserId, const char * szUserPassword, const char * szLoginCode, const char * szTicketType, unsigned short usOSType);

	virtual bool Logout();
	virtual bool Relogin();
	virtual void Disconnect();
	
	virtual const char * GetLastErrorMsg();

	virtual const char * GetResultCodeMsg(int nResultCode);

	virtual void Delete();

	virtual void SendMessage(unsigned int nUserDefinedMsgCode, const char * pMsg, int nSize, LPCWSTR strToUserID);
	virtual void ChangeOnlineStatus(E_MSGR_ONLINE_STATUS status, const char *pCustomDataBuffer = NULL, int nBufferSize = 0, bool bCustomDataDBSave = false);
	virtual void ChangeNick(LPCWSTR wszNick);

	virtual void RegiFavoriteToPOI(vector<wstring> &vecUsers, USHORT nUserGrp);
	virtual void UregFavoriteFromPOI(USHORT nUserGrp);
	virtual void UregFavoriteFromPOI(vector<wstring> &vecUsers, USHORT nUserGrp);
	virtual void GetBuddyStatus(std::vector<wstring> &vecUsers);

	//{ 2005/06/12 by chan
	virtual const char * GetServerDefinedCodeMsg(int nCode);

	virtual void InitOnlineStatus(E_MSGR_ONLINE_STATUS status);

	virtual int GetDnsFailReport() { return m_nDnsFailReport; }		// js α ÿ DNS fail  ־°?

	virtual void SetLoginServerAddress(LPCWSTR strAddress, UINT nPort);

	virtual void SendSessionControlPacket(void);
	virtual void SetPingSettting(bool bSendPing, int nInterval);
	virtual bool SendPing(void);
	virtual void RecoveryNetwork(void);


	////////////////////////////////////////////////////////////////////////////
	// class _INpSessionEvents
	virtual void CALLBACK OnNpSession_Connect(INpSession *pSession, int nSocketError);
	virtual void CALLBACK OnNpSession_Close(INpSession *pSession, int nSocketError, bool bRecvInvalidPacket);
	virtual void CALLBACK OnNpSession_RecvPacket(INpSession *pSession, INpPacket * pPacket);
	virtual void CALLBACK OnNpSession_SendReady(INpSession *pSession);

	void ParseLoginResponsePacket(LPCSTR szData, int nDataLen, bool bRelogin);	// login data  ̴ USHORT Ÿ ƴϴ. 65536  Ѿ ֱ ̴.

	bool OnTimerProcess(WPARAM wParam, LPARAM lParam);
	bool OnSignalProcess(int nMsg, WPARAM wParam, LPARAM lParam);

	
protected:
	virtual void EncryptString(LPCSTR szData, LPCSTR szSeed, BYTE &bytePwdCryptType, int &nPwdCryptKeyBaseSerial, BYTE biEncryptedPwd[], USHORT &usEncryptedPwdLength);
	virtual void EncryptString(const string &strData, const std::string& strSeed, BYTE &bytePwdCryptType, int &nPwdCryptKeyBaseSerial, BYTE biEncryptedPwd[], USHORT &usEncryptedPwdLength);
	virtual const char* GetAppandOSVersionString(char* szVersion, int nBufLen);
	virtual void GetEncryptUserPassword(BYTE &bytePwdCryptType, BYTE biEncryptedPwd[], USHORT &usEncryptedPwdLength);

	void ReadBuddyDataCacheSerial(ULONG &nBuddyListCacheSerial, ULONG &nGroupListCacheSerial );

	INpSession * m_pLoginNpSession;	
	vector<char> m_vecLoginResponseData;

	bool Internal_LoginConnect();

	void ParseLoginSuccessResponse( USHORT usServerResCode, CGBStream & stream );
	void ParseReloginSuccessResponse( USHORT usServerResCode, CGBStream & stream );
	void CALLBACK OnInternalLoginResult(USHORT usResultCode, USHORT usErrorCode);
	void CALLBACK OnInternalReloginResult(USHORT usResultCode, USHORT usErrorCode);

	INpSession * m_pMainServerNpSession;
	void CALLBACK OnInternalMainSessionClosed(int nSocketError);
	
	void StartCheckNetworkProcess();
	void StopCheckNetworkProcess();

	void StartReloginProcess(bool directConnectMode);
    void StopReloginProcess();
    
	void Send_LOGIN();

	void Send_RELOGIN();

	int m_nReloginTryCount;
	int m_nDnsFailReport;
	string	m_strLoginCode;
	string  m_strTicketType;
	string	m_strMobileNumber;
	string  m_strMobileToken;
	short m_nMobileDataOverwriteMode;
	
	bool m_bSendPing;
	int m_nSendPingInterval;
	bool m_bRecvPongFlag;
	uint32_t m_nLastRecvPongMSec;
	bool m_bPrevLoginOKFlag;
	bool m_bMobileFlag;
    bool m_bReloginProcess;
	USHORT m_uLangLocaleCode;
	uint16_t m_usOSType;
	
// class _CIkWndHandler
protected:

	INpSession * m_pNpSession_pool;
	void NpSession_AsyncDelete( INpSession * pNpSession )
	{
		if( m_pNpSession_pool )
			m_pNpSession_pool->Delete();

		if( pNpSession )
            pNpSession->CloseSession();

		m_pNpSession_pool = pNpSession;
	}

	//
	//class _INpSessionDispatcher_PacketHandler
	//
	virtual void CALLBACK INpSessionDispatcher_OnReceive_REQUEST_PACKET(INpSessionDispatcher * pEventSource, const INpPacket * pPacket);
	virtual void CALLBACK INpSessionDispatcher_OnReceive_RESPONSE_PACKET(INpSessionDispatcher * pEventSource, const INpPacket * pOriginalRequestPacket, const INpPacket * pPacket);

	virtual void CALLBACK INpSessionDispatcher_OnReceive_CToC(INpSessionDispatcher * pEventSource, USHORT usCToC_SubCMMD, const INpPacket * pPacket);
	
#if defined(WIN32)	// __KEH_USE_LKWND__
	virtual void CALLBACK IkWnd_OnMessage(CIkWnd * pIkWnd, UINT message, WPARAM wParam, LPARAM lParam );
#else
	virtual bool OnGTRunLoopObserver_SourceEvent(GT_EVENT_HANDLE handle, GT_PARAM param1, GT_PARAM param2);
	virtual bool OnGTRunLoopObserver_TimerEvent(GT_EVENT_HANDLE handle, GT_PARAM param);
#endif
	
private:
	E_MSGR_ONLINE_STATUS	m_eNewOnlineStatus;
	WSTRING					m_wstrNewNick;
	UINT					m_nLRSCookie;

	WSTRING					m_wstrLoginServerAddr;
	UINT					m_nLoginServerPort;
	string					m_strCustomData;
	
	nfc::CMutex				m_mutexSession;

	bool					m_bPrevNetworkCheckIsFalse;
};

const char * GetNpIMErrorMsg(int nNpErrorCode);
